#include "StdAfx.h"
#include "WASUpdateGroup.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"


CWASUpdateGroup::CWASUpdateGroup(CWASActor *pActor) : 
	CWASCommand(pActor)
{
    m_nGroupID = 0;
}


CWASUpdateGroup::~CWASUpdateGroup(void)
{
}


void CWASUpdateGroup::ExecuteAsync()
{
	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}

	USES_UTF8_CONVERSION;

	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/buddy/UpdateGroup.php");

	m_httpFacade.session().SetURL(W2U8(strURL.c_str()),	pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));
	
	char szID[64];
	sprintf(szID, "%d", m_nGroupID);
	m_httpFacade.session().AddBodyVariable("grpid", szID);
	m_httpFacade.session().AddBodyVariable("grpname", W2U8((LPCWSTR)m_sGroupInfo.strName.c_str()));
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}



bool CWASUpdateGroup::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	// ó XML !!		
	return true;
}


// override in drived class
void CWASUpdateGroup::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		OutputDebugString(_T("\r\n##### CWASUpdateGroup::OnWasResponse:true #####\r\n"));
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}
	else
	{
		if(StringUtil::StrCompareA(szWasResCodeInXml, "205") == 0)
			m_eError = wasError_DuplicateGroupName;
	}

	GT_ASSERT(m_pActor);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t observers;
	m_pActor->GetObserversEx(&observers);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t::iterator it = observers.begin();

	GT_BUDDY_GROUP_UPDATABLE_DATA sData;
	m_sGroupInfo.Convert(sData);

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEventPrivate *pObserver = static_cast<IGTWASActorEventPrivate *>(*it);
		if(pObserver->OnGTWASActor_ResponseUpdateGroup(m_pActor, GetID(), m_eError, m_nGroupID, sData) == false)
			break;
	}

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASUpdateGroup::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t observers;
	m_pActor->GetObserversEx(&observers);
	CGTObserved<IGTWASActorEventPrivate>::listobserver_t::iterator it = observers.begin();

	GT_BUDDY_GROUP_UPDATABLE_DATA sData;
	m_sGroupInfo.Convert(sData);

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEventPrivate *pObserver = static_cast<IGTWASActorEventPrivate *>(*it);
		if(pObserver->OnGTWASActor_ResponseUpdateGroup(m_pActor, GetID(), m_eError, m_nGroupID, sData) == false)
			break;
	}

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}
